/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/


/*
* ============================================================================
*  Name     : CContext_logContainer from Context_logContainer.h
*  Part of  : context_log
*  Created  : 13.2.2003 by 
*  Description:
*     Declares container control for application.
*  Version  :
*  Copyright: 
* ============================================================================
*/

#ifndef CONTEXT_LOGCONTAINER_H
#define CONTEXT_LOGCONTAINER_H

// INCLUDES
#include <coecntrl.h>

#include "i_logger.h"
#include "hash.h"
#include "log_base_impl.h"
#include "app_context.h"
#include "locationing.h"
#include "symbian_tree.h"
#include "circular.h"
   
#include "current_loc.h"
// FORWARD DECLARATIONS
class CEikLabel;        // for example labels


// CLASS DECLARATION

/**
*  CContext_logContainer  container control class.
*  
*/
class CContext_logContainer : public CCoeControl, MCoeControlObserver, public Mlogger, public MNaming, public MContextBase
    {
    public: // Constructors and destructor
        
        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
	CContext_logContainer(MApp_context& Context);
        void ConstructL(const TRect& aRect);

        /**
        * Destructor.
        */
        ~CContext_logContainer();

    public: // New functions
	void add_cellid_name(const TBBCellId& Cell, const TDesC& name);
	bool is_named(TInt id);
	void set_status(const TDesC& status);
	void set_error(const TDesC& err);
	CCircularLog*	BaseLog();
	CCurrentLoc* CurrentLoc();

    public: // Functions from base classes
	virtual void NewSensorEventL(const TTupleName& aName, const TDesC& aSubName, const CBBSensorEvent& aData);

    private: // Functions from base classes
	void register_source(const TDesC& name);

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;
	TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);

       /**
        * From ?base_class ?member_description
        */
        // event handling section
        // e.g Listbox events
        void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

    private: //data
        
        CEikLabel* iLabel;          // example label
	CEikLabel* status_label;
	CEikLabel* err_label;

	CDesC16Array *log_labels;

	CArrayFixFlat<CEikLabel*>* label_store;
	CArrayFixFlat<CEikLabel*>* value_store;

	CEikLabel*	find_value_label(const TDesC& name);

	CGenericIntMap* cellid_names;

	TBuf<100> current_name;

	TInt	iCursorPos;

	CCurrentLoc*	iCurrentLoc;
    };

#endif

// End of File
